#include "config.h"
#include <stdio.h>
#include <signal.h>

#include "error.h"
#include "options.h"
#include "uart.h"

struct Options opts;

static
void signal_handler(int sig)
{
  int exit_status;

  exit_status = options_deinit(&opts);
  exit(exit_status);
}

int main(int argc, char *argv[])
{
  int err = 0;
  int fd_uart;

  if (signal(SIGINT, signal_handler) == SIG_ERR)
    err_output("installing SIGINT signal handler");
  if (signal(SIGTERM, signal_handler) == SIG_ERR)
    err_output("installing SIGTERM signal handler");

  options_init(&opts);
  err = options_parse(&opts, argc, argv);
  if(err || opts.help)
  {
    options_usage(argv[0]);
    return err;
  }

  if(opts.verbose)
  {
    options_print(&opts);
  }

  speed_t speed = uart_get_speed(opts.speed);
  fd_uart = uart_setup(opts.ttyname, speed, opts.parity, opts.twostop);
  if(fd_uart < 0)
  {
    fprintf(stderr, "unable to setup UART\n");
    err = 3;
  }
  else
  {
    uart_poll(fd_uart, opts.verbose);
  }

  options_deinit(&opts);

  return err;
}
