#include "options.h"

int use_syslog = 0;

void options_usage(char *progname)
{
    struct Options opts;
    options_init(&opts);

    printf("Usage: %s [OPTIONS]\n", progname);
    printf("Version %s\n\n", VERSION);
    printf("Program description ....\n");
    printf("OPTIONS:\n\
        -h --help                Print help\n\
        -r --reset               SW Reset\n\
        -t --test                Perform a test\n\
        -s --speed               UART Speed\n\
        -p --parity              Enable even parity\n\
        -2 --twostop             Enable two stop bits\n\
        -f --tty-filename        Filename for the tty. Default is /dev/serial0\n\
        \n");

    options_deinit(&opts);
}

void options_init(struct Options *opts)
{
    opts->help = false;
    opts->verbose = false;
    opts->speed = -1;
    opts->parity = false;
    opts->twostop = false;
    snprintf(opts->ttyname, 64, "/dev/serial0");
}

int options_deinit(struct Options *opts)
{
    return 0;
}

int options_parse(struct Options *opts, int argc, char *argv[])
{
    int c, option_index, err = 0;
    static struct option long_options[] =
        {
            {"help", no_argument, 0, 'h'},
            {"verbose", no_argument, 0, 'v'},
            {"speed", required_argument, 0, 's'},
            {"parity", no_argument, 0, 'p'},
            {"twostop", no_argument, 0, '2'},
            {"tty-filename", no_argument, 0, 'f'},
            {0, 0, 0, 0}};

    while (1)
    {
        option_index = 0;
        c = getopt_long(argc, argv, "hvs:p2f:", long_options, &option_index);
        if (c == -1)
            break;
        switch (c)
        {
        case 0:
            if (strcmp("help", long_options[option_index].name) == 0)
                opts->help = true;
            if (strcmp("verbose", long_options[option_index].name) == 0)
                opts->verbose = true;
            if (strcmp("speed", long_options[option_index].name) == 0)
                opts->speed = atoi(optarg);
            if (strcmp("parity", long_options[option_index].name) == 0)
                opts->parity = true;
            if (strcmp("twostop", long_options[option_index].name) == 0)
                opts->twostop = true;
            if (strcmp("tty-filename", long_options[option_index].name) == 0)
                strncpy(opts->ttyname, optarg, 64);
            break;
        case 'h':
            opts->help = true;
            break;
        case 'v':
            opts->verbose = true;
            break;
        case 's':
            opts->speed = atoi(optarg);
            break;
        case 'p':
            opts->parity = true;
            break;
        case '2':
            opts->twostop = true;
            break;
        case 'f':
            strncpy(opts->ttyname, optarg, 64);
            break;
        default:
            err = 1;
            break;
        }

    }

    if (optind < argc)
    {
        err_output("non-option ARGV-elements: ");
        while (optind < argc)
            err_output("%s ", argv[optind++]);
        err_output("");
        err |= 1;
    }

    if(opts->speed == -1)
    {
      err |= 4;
      fprintf(stderr, "speed option not set\n");
    }

    return err;
}

void
options_print(struct Options *opts)
{
  printf("option verbose is %d\n", opts->verbose);
  printf("option help is %d\n", opts->help);
  printf("option speed is %d\n", opts->speed);
  printf("option parity is %d\n", opts->parity);
  printf("option twostop is %d\n", opts->twostop);
  printf("option tty-filename is %s\n", opts->ttyname);
}
